@@ -1,10 +1,11 @@
-from .action_client import ActionClient, create_action_client
-from .flows_client import FlowsClient, create_flows_client
+from .action_client import ActionClient
+from .client_helpers import create_action_client, create_flows_client
+from .flows_client import FlowsClient
 from .graphviz_rendering import graphviz_format, state_colors_for_log
 from .queues_client import QueuesClient, create_queues_client
 from .token_management import get_authorizer_for_scope
 
-__version__ = "0.10.1"
+__version__ = "0.10.5"
 
 __all__ = (
     "ActionClient",
