@@ -8,7 +8,9 @@ from globus_sdk import GlobusHTTPResponse
 from globus_automate_client.cli.callbacks import (
     flows_endpoint_envvar_callback,
     json_validator_callback,
-    principal_validator_callback,
+    principal_or_all_authenticated_users_validator,
+    principal_or_public_validator,
+    principal_validator,
     url_validator_callback,
 )
 from globus_automate_client.cli.helpers import (
@@ -16,10 +18,10 @@ from globus_automate_client.cli.helpers import (
     format_and_echo,
     verbosity_option,
 )
+from globus_automate_client.client_helpers import create_flows_client
 from globus_automate_client.flows_client import (
     PROD_FLOWS_BASE_URL,
     FlowValidationError,
-    create_flows_client,
     validate_flow_definition,
 )
 from globus_automate_client.graphviz_rendering import (
@@ -81,7 +83,8 @@ def flow_deploy(
         callback=json_validator_callback,
     ),
     subtitle: str = typer.Option(
-        None, help="A subtitle for the Flow providing additional, brief description.",
+        None,
+        help="A subtitle for the Flow providing additional, brief description.",
     ),
     description: str = typer.Option(
         None, help="A long form description of the Flow's purpose or usage."
@@ -103,18 +106,26 @@ def flow_deploy(
     ),
     visible_to: List[str] = typer.Option(
         None,
-        help="A principal which may see the existence of deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        help=(
+            "A principal which may see the existence of the deployed Flow. The "
+            'special value of "public" may be used to control which users can '
+            "discover this flow. [repeatable]"
+        ),
+        callback=principal_or_public_validator,
     ),
     administered_by: List[str] = typer.Option(
         None,
         help="A principal which may update the deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        callback=principal_validator,
     ),
     runnable_by: List[str] = typer.Option(
         None,
-        help="A principal which may run an instance of the deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        help=(
+            "A principal which may run an instance of the deployed Flow. The special "
+            'value of "all_authenticated_users" may be used to control which users '
+            "can invoke this flow. [repeatable]"
+        ),
+        callback=principal_or_all_authenticated_users_validator,
     ),
     validate: bool = typer.Option(
         True,
@@ -123,7 +134,9 @@ def flow_deploy(
         show_default=True,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -164,7 +177,8 @@ def flow_update(
         callback=json_validator_callback,
     ),
     subtitle: str = typer.Option(
-        None, help="A subtitle for the Flow providing additional, brief description.",
+        None,
+        help="A subtitle for the Flow providing additional, brief description.",
     ),
     description: str = typer.Option(
         None, help="A long form description of the Flow's purpose or usage."
@@ -186,18 +200,26 @@ def flow_update(
     ),
     visible_to: List[str] = typer.Option(
         None,
-        help="A principal which may see the existence of deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        help=(
+            "A principal which may see the existence of the deployed Flow. The "
+            'special value of "public" may be used to control which users can '
+            "discover this flow. [repeatable]"
+        ),
+        callback=principal_or_public_validator,
     ),
     administered_by: List[str] = typer.Option(
         None,
         help="A principal which may update the deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        callback=principal_validator,
     ),
     runnable_by: List[str] = typer.Option(
         None,
-        help="A principal which may run an instance of the deployed Flow. [repeatable]",
-        callback=principal_validator_callback,
+        help=(
+            "A principal which may run an instance of the deployed Flow. The special "
+            'value of "all_authenticated_users" may be used to control which users '
+            "can invoke this flow. [repeatable]"
+        ),
+        callback=principal_or_all_authenticated_users_validator,
     ),
     validate: bool = typer.Option(
         True,
@@ -206,7 +228,9 @@ def flow_update(
         show_default=True,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -293,8 +317,24 @@ def flow_list(
         case_sensitive=False,
         show_default=True,
     ),
+    marker: str = typer.Option(
+        None,
+        "--marker",
+        "-m",
+        help="A pagination token for iterating through returned data.",
+    ),
+    per_page: int = typer.Option(
+        None,
+        "--per-page",
+        "-p",
+        help="The page size to return. Only valid when used without providing a marker.",
+        min=1,
+        max=50,
+    ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -302,7 +342,9 @@ def flow_list(
     List Flows for which you have access.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
-    flows = fc.list_flows(roles=[r.value for r in roles])
+    flows = fc.list_flows(
+        roles=[r.value for r in roles], marker=marker, per_page=per_page
+    )
     format_and_echo(flows, verbose=verbose)
 
 
@@ -318,7 +360,9 @@ def flow_display(
         show_default=True,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -343,7 +387,9 @@ def flow_delete(
         show_default=True,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -373,7 +419,9 @@ def flow_run(
         callback=url_validator_callback,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
@@ -392,7 +440,9 @@ def flow_run(
 @app.command("action-list")
 def flow_actions_list(
     flow_id: str = typer.Option(
-        ..., help="The ID for the Flow which triggered the Action.", prompt=True,
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -405,15 +455,33 @@ def flow_actions_list(
         help="Display Actions where you have the selected role. [repeatable]",
     ),
     statuses: List[ActionStatus] = typer.Option(
-        None, "--status", help="Display Actions with the selected status. [repeatable]",
+        None,
+        "--status",
+        help="Display Actions with the selected status. [repeatable]",
+    ),
+    marker: str = typer.Option(
+        None,
+        "--marker",
+        "-m",
+        help="A pagination token for iterating through returned data.",
+    ),
+    per_page: int = typer.Option(
+        None,
+        "--per-page",
+        "-p",
+        help="The page size to return. Only valid when used without providing a marker.",
+        min=1,
+        max=50,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
     """
-    List the Actions associated with a Flow.
+    List a Flow definition's discrete invocations.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
 
@@ -427,7 +495,12 @@ def flow_actions_list(
         roles_str = [r.value for r in roles]
 
     action_list = fc.list_flow_actions(
-        flow_id, flow_scope, statuses=statuses_str, roles=roles_str
+        flow_id,
+        flow_scope,
+        statuses=statuses_str,
+        roles=roles_str,
+        marker=marker,
+        per_page=per_page,
     )
     format_and_echo(action_list, verbose=verbose)
 
@@ -436,7 +509,9 @@ def flow_actions_list(
 def flow_action_status(
     action_id: str = typer.Argument(...),
     flow_id: str = typer.Option(
-        ..., help="The ID for the Flow which triggered the Action.", prompt=True,
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -444,12 +519,14 @@ def flow_action_status(
         callback=url_validator_callback,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
     """
-    Retrieve a Flow's Action's status.
+    Display the status for a Flow definition's particular invocation.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     response = fc.flow_action_status(flow_id, flow_scope, action_id)
@@ -460,7 +537,9 @@ def flow_action_status(
 def flow_action_release(
     action_id: str = typer.Argument(...),
     flow_id: str = typer.Option(
-        ..., help="The ID for the Flow which triggered the Action.", prompt=True,
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -468,12 +547,14 @@ def flow_action_release(
         callback=url_validator_callback,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
     """
-    Retrieve a Flow's Action's status.
+    Remove execution history for a particular Flow definition's invocation.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     response = fc.flow_action_release(flow_id, flow_scope, action_id)
@@ -484,7 +565,9 @@ def flow_action_release(
 def flow_action_cancel(
     action_id: str = typer.Argument(...),
     flow_id: str = typer.Option(
-        ..., help="The ID for the Flow which triggered the Action.", prompt=True,
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -492,12 +575,14 @@ def flow_action_cancel(
         callback=url_validator_callback,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
     """
-    Retrieve a Flow's Action's status.
+    Cancel an active execution for a particular Flow definition's invocation.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
     response = fc.flow_action_cancel(flow_id, flow_scope, action_id)
@@ -508,7 +593,9 @@ def flow_action_cancel(
 def flow_action_log(
     action_id: str = typer.Argument(...),
     flow_id: str = typer.Option(
-        ..., help="The ID for the Flow which triggered the Action.", prompt=True,
+        ...,
+        help="The ID for the Flow which triggered the Action.",
+        prompt=True,
     ),
     flow_scope: str = typer.Option(
         None,
@@ -522,7 +609,24 @@ def flow_action_log(
         help="Display logs starting from most recent and proceeding in reverse chronological order",
     ),
     limit: int = typer.Option(
-        None, help="Set a maximum number of events from the log to return", min=0,
+        None,
+        help="Set a maximum number of events from the log to return",
+        min=0,
+        max=100,
+    ),
+    marker: str = typer.Option(
+        None,
+        "--marker",
+        "-m",
+        help="A pagination token for iterating through returned data.",
+    ),
+    per_page: int = typer.Option(
+        None,
+        "--per-page",
+        "-p",
+        help="The page size to return. Only valid when used without providing a marker.",
+        min=1,
+        max=50,
     ),
     output_format: FlowDisplayFormat = typer.Option(
         FlowDisplayFormat.json,
@@ -533,15 +637,19 @@ def flow_action_log(
         show_default=True,
     ),
     flows_endpoint: str = typer.Option(
-        PROD_FLOWS_BASE_URL, hidden=True, callback=flows_endpoint_envvar_callback,
+        PROD_FLOWS_BASE_URL,
+        hidden=True,
+        callback=flows_endpoint_envvar_callback,
     ),
     verbose: bool = verbosity_option,
 ):
     """
-    Get a log of the steps executed by a Flow's Action.
+    Get a log of the steps executed by a Flow definition's invocation.
     """
     fc = create_flows_client(CLIENT_ID, flows_endpoint)
-    resp = fc.flow_action_log(flow_id, flow_scope, action_id, limit, reverse)
+    resp = fc.flow_action_log(
+        flow_id, flow_scope, action_id, limit, reverse, marker, per_page
+    )
 
     if verbose:
         display_http_details(resp)
